"""import records

# Record types
RECORD_TYPES = {
    "host": records.Host,
    "interface": records.Interface,
    "person": records.Person,
    "service": records.Service
}"""

# Record Keys
LS_KEY_TYPE = "type"
LS_KEY_EXPIRES = "expires"
LS_KEY_TTL = "ttl"
LS_KEY_URI = "uri"
LS_KEY_STATE = "state"

#Type Values
LS_VALUE_TYPE_SERVICE = "service"
LS_VALUE_TYPE_HOST = "host"
LS_VALUE_TYPE_INTERFACE = "interface"
LS_VALUE_TYPE_PERSON = "person"

#State Values
LS_VALUE_STATE = ["registered", "renewed", "deleted", "expired"]

# Operator Keys
LS_KEY_OPERATOR = "operator"
LS_KEY_OPERATOR_SUFFIX = "-operator"

# Operator Values
LS_VALUES_OPERATOR = ["any", "all"]

# Location Keys
LS_KEY_LOCATION_SITENAME = "location-sitename"
LS_KEY_LOCATION_CITY = "location-city"
LS_KEY_LOCATION_REGION = "location-region"
LS_KEY_LOCATION_STATE = "location-state"
LS_KEY_LOCATION_COUNTRY = "location-country"
LS_KEY_LOCATION_CODE = "location-code"
LS_KEY_LOCATION_LATITUDE = "location-latitude"
LS_KEY_LOCATION_LONGITUDE = "location-longitude"

# Group Keys
LS_KEY_GROUP_COMMUNITIES = "group-communities"
LS_KEY_GROUP_DOMAINS = "group-domains"

# Service Keys
LS_KEY_SERVICE_NAME = "service-name"
LS_KEY_SERVICE_TYPE = "service-type"
LS_KEY_SERVICE_VERSION = "service-version"
LS_KEY_SERVICE_HOST = "service-host"
LS_KEY_SERVICE_LOCATOR = "service-locator"
LS_KEY_SERVICE_ADMINISTRATORS = "service-administrators"

# Host Keys
LS_KEY_HOST_NAME = "host-name"
LS_KEY_HOST_HARDWARE_MEMORY = "host-hardware-memory"
LS_KEY_HOST_HARDWARE_PROCESSORSPEED = "host-hardware-processorspeed"
LS_KEY_HOST_HARDWARE_PROCESSORCOUNT = "host-hardware-processorcount"
LS_KEY_HOST_HARDWARE_PROCESSORCORE = "host-hardware-processorcore"

LS_KEY_HOST_OS_NAME = "host-os-name"
LS_KEY_HOST_OS_VERSION = "host-os-version"
LS_KEY_HOST_OS_KERNEL = "host-os-kernel"

LS_KEY_HOST_NET_TCP_CONGESTIONALGORITHM = "host-net-tcp-congestionalgorithm"
LS_KEY_HOST_NET_TCP_MAXBUFFER_SEND = "host-net-tcp-maxbuffer-send"
LS_KEY_HOST_NET_TCP_MAXBUFFER_RECV = "host-net-tcp-maxbuffer-recv"
LS_KEY_HOST_NET_TCP_AUTOTUNEMAXBUFFER_SEND = "host-net-tcp-autotunemaxbuffer-send"
LS_KEY_HOST_NET_TCP_AUTOTUNEMAXBUFFER_RECV = "host-net-tcp-autotunemaxbuffer-recv"
LS_KEY_HOST_NET_TCP_MAXBACKLOG = "host-net-tcp-maxbacklog"

LS_KEY_HOST_NET_INTERFACES = "host-net-interfaces"
LS_KEY_HOST_ADMINISTRATORS = "host-administrators"

# Interface Keys
LS_KEY_INTERFACE_NAME = "interface-name"
LS_KEY_INTERFACE_ADDRESSES = "interface-addresses"
LS_KEY_INTERFACE_SUBNET = "interface-subnet"
LS_KEY_INTERFACE_CAPACITY = "interface-capacity"
LS_KEY_INTERFACE_MAC = "interface-mac"
LS_KEY_INTERFACE_MTU = "interface-mtu"

# Person Keys
LS_KEY_PERSON_NAME = "person-name"
LS_KEY_PERSON_EMAILS = "person-emails"
LS_KEY_PERSON_PHONENUMBERS = "person-phonenumbers"
LS_KEY_PERSON_ORGANIZATION = "person-organization"



__author__ = 'sowmya'

from sls_client.records import *
from sls_client.query import *
import sys
import json
import socket

from optparse import OptionParser

def get_ma_for_host(hostname):

    ipaddresses={}
    if(not hostname):
        raise Exception("Null Parameter", "Hostname parameter is empty.")
    ipaddr = socket.getaddrinfo(hostname, None)

    #ipaddr is a 5-tuple and the ipv4 or ipv6 is the last element in the tuple.
    for ip in ipaddr:
        ipaddresses[ip[4][0]]=1

    response=[]
    result={}
    queryKeys = ['psmetadata-dst-address','psmetadata-src-address']
    for queryKey in queryKeys:
        queryString=""
        for ip in ipaddresses:
            queryString += queryKey+"="+ip+"&"
        response += query(queryString)


    for record in response:
        malocators = record[u'psmetadata-ma-locator']
        for malocator in malocators:
            malocatorString = malocator.encode("ascii")
            result[malocatorString] = 1

    return result.keys()

def get_json(query,hostlist):
    result={}
    result["ma-hosts"] = []
    result["search-query"] = query
    if(hostlist):
        result["ma-hosts"] = hostlist

    json_output = json.dumps(result)
    return json_output


def main():

    parser = OptionParser()

    parser.add_option("-n", "--hostname",
                      dest="hostname",
                      help="specify hostname or IP")
    parser.add_option("-o", "--output",
                      dest="output_type",
                      help="output type - json or console",
                      choices=["console","json"],
                      default="console"
                      )
    (options, args) = parser.parse_args()

    if (not options.hostname):
        print "Please specify hostname"
        sys.exit(1)

    result = get_ma_for_host(options.hostname)
    if(options.output_type == "json"):
        output=get_json(options.hostname,result)
        print output
    else:
        for host in result:
            print host
if __name__=='__main__':
    main()


from datetime import datetime, timedelta
import re
import warnings

from isodate import parse_datetime, parse_duration
from isodate import datetime_isoformat, duration_isoformat

from voluptuous import All, Any, Length, Range
from voluptuous import Invalid, MultipleInvalid

from constants import *

LS_MAC_ADDRESS_FORMAT = re.compile(r"^([0-9A-F]{2}[-]){5}([0-9A-F]{2})$|^([0-9A-F]{2}[:]){5}([0-9A-F]{2})$", re.IGNORECASE)
LS_COUNTRY_FORMAT = re.compile(r"^[A-Z]{2}$", re.IGNORECASE)

class LSRecordInfoWarning(Warning): pass
class LSTimestampWarning(LSRecordInfoWarning): pass
class LSDurationWarning(LSRecordInfoWarning): pass
class LSStateWarning(LSRecordInfoWarning): pass
class LSCountryWarning(LSRecordInfoWarning): pass
class LSMACAddressWarning(LSRecordInfoWarning): pass
class LSLatitudeWarning(LSRecordInfoWarning): pass
class LSLongitudeWarning(LSRecordInfoWarning): pass

def LSBase(msg=None):
    def f(v):
        if isinstance(v, basestring):
            v = unicode(v)
        else:
            raise Invalid(msg or "expected string")
        return v
    return f

def LSKey(validator=None, msg=None):
    def f(v):
        if not v.isalnum():
            raise Invalid(msg or "key must be alphanumeric")
        return v
    base = LSBase(msg="key must be a string")
    if validator:
        return All(base, f, validator)
    return All(base, f)

def LSValue(validator=None, msg=None):
    def f(v):
        if isinstance(v, list):
            if len(v) == 1:
                return v[0]
            elif len(v) == 0:
                return unicode("")
            else:
                raise Invalid(msg or "expected single value")
        return v
    base = LSBase(msg="value must be a string")
    if validator:
        return All(f, validator, base)
    return All(f, base)

def LSValues(validator=None, msg=None):
    def f(v):
        if not isinstance(v, list):
            return [v]
        return v
    return All(f, All([LSValue(validator)], Length(min=1)))

def LSTimestamp():
    def f(v):
        if isinstance(v, datetime):
            v = datetime_isoformat(v)
        else:
            try:
                v = datetime_isoformat(parse_datetime(v))
            except:
                warnings.warn("incorrect timestamp format", LSTimestampWarning)
        return v
    return f

def LSDuration():
    def f(v):
        if isinstance(v, timedelta):
            v = duration_isoformat(v)
        else:
            try:
                v = duration_isoformat(parse_duration(v))
            except:
                warnings.warn("incorrect duration format", LSDurationWarning)
        if isinstance(v, basestring) and v.startswith("-"):
            warnings.warn("duration should not be negative", LSDurationWarning)
        return v
    return f

def LSState():
    pass

def LSCountry():
    def f(v):
        if not isinstance(v, basestring) or not LS_COUNTRY_FORMAT.match(v):
            warnings.warn("incorrect country format", LSCountryWarning)
    return f

def LSLatitude():
    def f(v):
        longitude = None
        try:
            latitude = float(v)
        except:
            pass
        if latitude is None or math.abs(latitude) > 90:
            warnings.warn("incorrect latitude format", LSLatitudeWarning)
    return f

def LSLongitude():
    def f(v):
        longitude = None
        try:
            longitude = float(v)
        except:
            pass
        if longitude is None or math.abs(longitude) > 180:
            warnings.warn("incorrect longitude format", LSLongitudeWarning)
    return f

def LSMACAddress():
    def f(v):
        if not isinstance(v, basestring) or not LS_MAC_ADDRESS_FORMAT.match(v):
            warnings.warn("incorrect MAC address format", LSMACAddressWarning)
    return f

def LSNetworkAddress():
    pass



import copy
import json

from voluptuous import Schema
from voluptuous import Extra, Required

from constants import *
from validators import *

class Record(object):
    """Base class for record objects"""
    
    record_schema = Schema({
        Required(LSKey(LS_KEY_TYPE)): LSValues(),
        LSKey(LS_KEY_EXPIRES): LSValues(),
        LSKey(LS_KEY_TTL): LSValues(),
        LSKey(LS_KEY_URI): LSValues(),
        LSKey(LS_KEY_STATE): LSValues(),
        LSKey(LS_KEY_GROUP_COMMUNITIES): LSValues(),
        LSKey(LS_KEY_GROUP_DOMAINS): LSValues(),
        LSKey(LS_KEY_LOCATION_SITENAME): LSValues(),
        LSKey(LS_KEY_LOCATION_CITY): LSValues(),
        LSKey(LS_KEY_LOCATION_REGION): LSValues(),
        LSKey(LS_KEY_LOCATION_STATE): LSValues(),
        LSKey(LS_KEY_LOCATION_COUNTRY): LSValues(),
        LSKey(LS_KEY_LOCATION_CODE): LSValues(),
        LSKey(LS_KEY_LOCATION_LATITUDE): LSValues(),
        LSKey(LS_KEY_LOCATION_LONGITUDE): LSValues(),
        Extra: LSValues()
    })
    
    def __init__(self, data):
        super(Record, self).__init__()
        data = self.record_schema(dict(data))
        super.__setattr__(self, "_data", data)
    
    @property
    def data(self):
        return copy.deepcopy(self._data)
    
    def dump(self, pretty=False):
        if pretty:
            return json.dumps(self._data, sort_keys=True, indent=4)
        else:
            return json.dumps(self._data)
    
    @property
    def pretty(self):
        return self.dump(True)
    
    @property
    def raw(self):
        return self.dump(False)
    
    #TODO: implement delete
    def __getattr__(self, name):
        return self.__getitem__(name.replace("_", "-"))
        
    def __setattr__(self, name, value):
        self.__setitem__(name.replace("_", "-"), value)
    
    def __getitem__(self, name):
        return self._data.__getitem__(name)
    
    def __setitem__(self, name, value):
        self._data.__setitem__(name, value)

class Host(Record):
    """Base class for host record objects"""
    
    def __init__(self, data):
        pass
    
class Interface(Record):
    """Base class for interface record objects"""
    
    def __init__(self, data):
        pass
    
class Person(Record):
    """Base class for person record objects"""
    
    def __init__(self, data):
        pass

class Service(Record):
    """Base class for service record objects"""
    
    def __init__(self, data):
        pass
